; $id$
;
; License: GPL-3

(declare (export))
(use svnwiki-extensions-support svn-client posix srfi-40 stream-ext html-stream srfi-1 iconv)

(define (svnwiki-upload-with-dir env func)
  (let-from-environment env (path-in path)
    (if (and (file-exists? (svnwiki-make-pathname path-in path))
             (directory? (svnwiki-make-pathname path-in path)))
      (func env)
      (svnwiki-upload-with-dir (environment env ((path (svnwiki-make-pathname (butlast (string-split path "/")))))) func))))

(define *svnwiki-upload-javascript*
#<<EOF
if (typeof(svnwikiUpload) != "object") {
var svnwikiUpload = new Object();

svnwikiUpload.submit = function ()
{
  document.getElementById("svnwikiUploadForm").style.display = 'none';
  document.getElementById("svnwikiUploadWait").style.display = 'inline';
  document.getElementById("svnwikiUploadWaitFocus").focus();
}

} // if
EOF
)

(define (svnwiki-upload-form env)
  (svnwiki-upload-with-dir env
    (lambda (env)
      (let-from-environment env (path-in path)
        (render-template
          env
          "Upload File: "
          (html-stream
            ((script type "text/javascript") *svnwiki-upload-javascript*)
            ((a name "svnwikiUploadWaitFocus" id "svnwikiUploadWaitFocus"))
            ((div id "svnwikiUploadWait" style "display: none;")
             (h2 "Upload in progress")
             (p "Your file is currently being uploaded.")
             (p "Please leave this window open until your upload is finished."))
            ((form method "post"
                   action "?action=extension&extension=upload&upload-contents=yes"
                   enctype "multipart/form-data"
                   id "svnwikiUploadForm"
                   onSubmit "svnwikiUpload.submit();")
             (h2 "Upload")
             (p (b "Name:")
                " The name you want to use for the file"
                (if (string=? path "")
                  stream-null
                  (html-stream
                    " (relative to path "
                    (tt ((a href ".") path))
                    ")"))
                "."
                (br)
                ((input type "text" name "filename")))
             (p (b "File:")
                (br)
                ((input type "file" name "file")))
             (svnwiki-interface-stream-form-meta env #f)
             (p ((input type "submit" value "Upload")))))
          'upload)))))

(define (svnwiki-upload-name env)
  (let-from-environment env (user-input)
    (let* ((name (svnwiki-upload-trim-extension
                   (stream->string
                     (stream-downcase
                       (stream-last
                         (stream-split
                           (user-input 'filename (user-input 'file.filename stream-null))
                           (cut char=? #\/ <>))))))))
      (if (string=? name "")
        (format #f "upload.~A" (current-seconds)) ; What else?
        name))))

; List of extensions (ie. suffixes for the file names) that
; Svnwiki will strip from the file name.

(define *svnwiki-upload-extensions*
  '(html png htm jpeg jpg doc txt gif css js odt ods odp xls pdf tar gz pl scm))

(define (svnwiki-upload-trim-extension name)
  (let loop ((index -1))
    (cond
      ((string-index name #\. (+ index 1)) => loop)
      ((and (not (negative? index))
            (member (string->symbol (substring name (+ index 1))) *svnwiki-upload-extensions*))
       (svnwiki-upload-trim-extension (substring name 0 index)))
      (else name))))

(define (svnwiki-upload-contents env)
  (let-from-environment env (user-input path-in path user password)
    (let* ((name (svnwiki-upload-name env))
           (full-path (svnwiki-repository-path env name))
           (new (not (file-exists? full-path)))
           (type (user-input 'file.content-type (string->stream "application/octet-stream"))))
      (cond
        ((directory? full-path)
         (svnwiki-upload-contents
           (let ((new-user-input (lambda (op . rest)
                                   (if (eq? op 'filename)
                                     (string->stream (format #f "~A.~A" name (current-seconds)))
                                     (apply user-input op rest)))))
             (environment env ((user-input new-user-input))))))
        ((or (is-frozen? path-in path)
             (not (get-props-parents-first-boolean env "svnwiki:upload:enabled" #t)))
         (svnwiki-error-html env
           (html-stream "You can not upload a file to that location.")))
        ((not (svnwiki-edit-question-verify env))
         (svnwiki-error-html env
           (html-stream "Ooops, you didn't provide a valid answer to the spam challenge.")))
        ((not (or new
                  (get-props-parents-first-boolean env "svnwiki:upload:allow-overwrite" #t)))
         (svnwiki-error-html env
           (html-stream "That file already exists and overwriting existing files is not allowed.")))
        (else
          (when new
            (svnwiki-create-add-parents env))
          (with-output-to-file full-path
            (lambda ()
              (write-stream (user-input 'file stream-null))))
          (when new
            (svn-add full-path))
          (cond
            ((not (svn-client-propset "svn:mime-type" (svn-string-create (stream->string type)) full-path #f #f))
             (svnwiki-error-html env
               (html-stream "Invalid MIME type: " (i type))))
            ((not (svn-commit
                    path-in
                    user
                    password
                    (format #f "Uploading file for user ~A (~A)"
                            (iconv (iconv-open "us-ascii" "us-ascii")
                                   (stream->string (get-commit-author env (string->stream "Anonymous"))))
                            (getenv "REMOTE_ADDR"))))
             (svnwiki-error-html env
               (html-stream "Ooops, commit failed.")))
            (else
             (render-template
               env
               "Upload successful: "
               (html-stream
                 (h2 "Upload successful")
                 (p "File " (i name) " (having " (stream-length (user-input 'file stream-null)) " bytes "
                    "and type " (i type) ") "
                    "has been added to the repository."))
               'upload))))))))
(define (svnwiki-upload-dynamic env)
  (->stream-char
    "Content-type: text/html\n\n"
    (let-from-environment env (user-input)
      (if (stream= char=? (user-input 'upload-contents stream-null) (string->stream "yes"))
        (svnwiki-upload-contents env)
        (svnwiki-upload-form env)))))

(define (svnwiki-upload-action-link env)
  (let-from-environment env (path-in path application-url)
    (when (and (or (directory? (svnwiki-make-pathname path-in path))
                   (string=? "index" (last (string-split path "/"))))
               (not (svnwiki-is-discuss? path))
               (get-props-parents-first-boolean env "svnwiki:upload:enabled" #t)
               (get-props-parents-first-boolean env "svnwiki:links:upload" #t))
      (svnwiki-file-action-link
        env
        (format #f "~A/~A?action=extension&extension=upload"
                application-url
                path)
        "Upload"))))

(svnwiki-extension-define 'dynamic 'upload svnwiki-upload-dynamic)
(svnwiki-extension-define 'files-actions-links 'upload svnwiki-upload-action-link)
